﻿using Newtonsoft.Json;
using System;
using System.Net;
using UnityEngine;
using WebSocketSharp;
using WebSocketSharp.Server;

namespace TeachR.WebCommunication
{
    public class SocketServer : WebSocketBehavior
    {
        private static WebSocketServer wssv;
        private static SocketServer _server;

        public static event Action<string, string> OnIncomingMessage;
        public static event Action<string, object> OnOutgoingMessage;
        internal static event Action OnRtcOpen, OnRtcClose;

        protected override void OnMessage(MessageEventArgs e)
        {
            char[] sep = { ';' };
            string[] action = e.Data.Split(sep, 2);
            OnIncomingMessage?.Invoke(action[0], action[1]);
        }

        protected override void OnOpen()
        {
            OnRtcOpen?.Invoke();
        }

        [Obsolete]
        public static SocketServer HostServer(IPAddress hostIp, int port, string servicePath)
        {
            wssv = new WebSocketServer(hostIp, port);
            _server = new SocketServer();

            // FIXME deprecated, but I can't find documentation on how to properly deal with this
            wssv.AddWebSocketService("/" + servicePath, () => _server);
            wssv.Start();
            Debug.Log($"[socket] Start connection at {hostIp}:{port}/{servicePath}");

            return _server;
        }

        public static void CloseServer()
        {
            OnRtcClose?.Invoke();
            wssv.Stop();
        }

        internal static void Send(string type, object payload)
        {
            string serialized = JsonConvert.SerializeObject(new Msg { type = type, payload = payload });
            _server.Send(serialized);
            OnOutgoingMessage?.Invoke(type, payload);
        }

        private class Msg
        {
            public string type;
            public object payload;
        }
    }
}