using System;
using UnityEngine;

namespace TeachR.WebCommunication
{
    /// <summary>
    /// Sync the position of the object this component is attached to.
    /// </summary>
    public class SyncPosition2D : MonoBehaviour
    {
        /// <summary>
        /// Rate of the sync actions.
        /// </summary>
        public int frequency = 10; // 10 FPS

        private Transform _transform;

        [SerializeField]
        private string id;

        public string Id => id != "" ? id : name;

        private void Awake()
        {
            _transform = transform;
        }

        private void OnEnable() => SyncPositionSystem.Add(this);
        private void OnDisable() => SyncPositionSystem.Remove(this);

        public Position2D ToPosition()
        {
            return new Position2D(_transform);
        }
    }

    [Serializable]
    public class Position2D
    {
        public float x, z, facingX, facingZ;

        public Position2D(Transform t)
        {
            Vector3 position = t.position;
            x = position.x;
            z = position.z;

            // TODO: Should this be an angle instead?
            Vector3 forward = t.forward;
            facingX = x + forward.x;
            facingZ = z + forward.z;
        }
    }
}