using System.Collections.Generic;
using UnityEngine;

namespace TeachR.WebCommunication
{
    public class SyncPositionSystem
    {
        private static readonly List<SyncPosition2D> Components = new();

        private const int RateOfUpdates = 10; // 10 FPS

        public static void Add(SyncPosition2D position) => Components.Add(position);
        public static void Remove(SyncPosition2D position) => Components.Remove(position);

        [RuntimeInitializeOnLoadMethod]
        private static void Init()
        {
            CoachRtc.RegisterOnReady(() =>
            {
                WebsocketDelegateHooks.OnUpdate += UpdatePositions;
            });
        }

        private static float _timer;

        private static void UpdatePositions()
        {
            _timer += Time.deltaTime;

            if (_timer < 1f / RateOfUpdates || Components.Count == 0) return;
            _timer -= 1f / RateOfUpdates;

            /* TODO: Update frontend to allow this
            var objects = Components.ToDictionary(pos => pos.Id, pos => pos.ToPosition());
            CoachRtc.Send("syncObjects", objects);
            */

            CoachRtc.Send("syncTeacher", Components[0].ToPosition());
        }
    }

}
